/*
 * AZ_CallerID for HS3
 * by A.A. van Zoelen
 * versie 1.0 - 2015
 * 
 * Return codes
 * 0 - Oke
 * 1 - HS3 unreachable
 * 2 - Fritz!Box unreachable
 * 3 - Telnet connection lost
 * 
 */
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;

namespace AZ_CallerID
{

    class Program
    {

        static int Main(string[] args)
        {
            // READ SETTINGS
            //=====================================================
            string HOST         = ConfigurationManager.AppSettings["HOST"];
            int PORT            = Convert.ToInt32(ConfigurationManager.AppSettings["PORT"] );
            bool AutoUpdatePB   = Convert.ToBoolean(ConfigurationManager.AppSettings["AutoUpdatePhonebook"]);
            //=====================================================

            string CallerName = ConfigurationManager.AppSettings["UnknownCaller"];
            string Unknown = CallerName;
            string errorText = string.Empty;
            TelnetConnection tc = new TelnetConnection(HOST, PORT);
            var tools = new Tools();
            string[] parts;

            int result = tools.HSConnect();
            if (result == 1)
            {
                string HSLocation = ConfigurationManager.AppSettings["HSLocation"];
                int HSPort = Convert.ToInt32(ConfigurationManager.AppSettings["HSPort"]);
                errorText = "Can not reach HS3 server at " + HSLocation + ":" + HSPort.ToString();
                Console.WriteLine(errorText);
                tools.WriteToLog(errorText);
                return 1;
            }
                        
            if (tc.IsConnected == true)
            {
                Console.WriteLine("AZ_CallerID for HS3 v1.0");
                Console.WriteLine("Now connected to {0}:{1}", HOST, PORT);
                Console.WriteLine("Listening for calls...");
                tools.WriteToLog(string.Format("{0} - AZ_CallerID started.", DateTime.Now) );
            }
            else
            {
                errorText = "Can not connect to the Fritz!Box at " + HOST + ":" + PORT.ToString();
                Console.WriteLine( errorText );
                tools.WriteToLog( errorText );
                return 2;
            }


            // while connected to the fritz!box
            while (tc.IsConnected )
            {
                string teststring = tc.Read();
                teststring.Replace(";;", ";0;");

                if (teststring.Contains(";"))
                {
                    Console.WriteLine("{0} - {1}", DateTime.Now, teststring);
                }

                // When disconnecting this is shown
                // 01.01.00 01:57:34;DISCONNECT;0;0;
                if (teststring.Contains("DISCONNECT"))
                {
                    tools.WriteToLog(string.Format("{0} - Disconnect", DateTime.Now) );
                    tools.DoDevice(0, "Verbinding verbroken");
                }

                // Split the incomming sting into parts starting at parts[0]
                parts = teststring.Split(';');

                // Incoming call
                // When a call comes in then this is shown
                // 01.01.00 01:57:23;RING;0;0612345678;0712345678;POTS;
                if(teststring.Contains("RING"))
                {
                    string CallerNr = parts[3];
                    CallerName = tools.ReadPhonebook(CallerNr);
                    tools.WriteToLog(string.Format("{0}",CallerName));
                    if (CallerName == Unknown)
                    {
                        CallerName = tools.FindName(CallerNr);
                        if (CallerName == "ERROR")
                        {
                            string URLNumberChecker = ConfigurationManager.AppSettings["URLNumberChecker"];
                            errorText = "Can't reach number resolving URL at " + URLNumberChecker + CallerNr;
                            Console.WriteLine(errorText);
                            tools.WriteToLog(errorText);
                        }
                        else
                        {
                            if ((CallerName != Unknown) && (AutoUpdatePB == true))
                            {
                                tools.UpdatePhonebook(CallerNr, CallerName);
                            }
                        }
                    }
                    tools.WriteToLog(string.Format("{0} - Inbound: Caller {1} - {2}", DateTime.Now, CallerNr, CallerName));
                    tools.DoDevice(Convert.ToInt32(CallerNr), CallerName);
                }

                // Outgoing call
                // 03.01.00 02:54:49;CALL;0;0;0712345678;0612345678;POTS;
                if (teststring.Contains("CALL"))
                {
                    string CallerNr = parts[5];
                    CallerName = tools.ReadPhonebook(CallerNr);
                    if (CallerName == Unknown)
                    {
                        CallerName = tools.FindName(CallerNr);
                        if (CallerName == "ERROR")
                        {
                            string URLNumberChecker = ConfigurationManager.AppSettings["URLNumberChecker"];
                            errorText = "Can't reach number resolving URL at " + URLNumberChecker + CallerNr;
                            Console.WriteLine(errorText);
                            tools.WriteToLog(errorText);
                        }
                        else
                        {
                            if ((CallerName != Unknown) && (AutoUpdatePB == true))
                            {
                                tools.UpdatePhonebook(CallerNr, CallerName);
                            }
                        }
                    }
                    tools.WriteToLog(string.Format("{0} - Outbound: Calling to number {1} - {2}", DateTime.Now, CallerNr, CallerName ));
                }

                // When incoming or outgoing connects
                // 03.01.00 02:57:52;CONNECT;0;0;0612345678;
                if (teststring.Contains(";CONNECT"))
                {
                    tools.WriteToLog(string.Format("{0} - Connected with {1}", DateTime.Now, parts[4]));
                }
             }
        return 3; // Telnet connection lost
        }
    }
}
