﻿using System;
using System.Net;
using System.IO;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
// HS3 related
using HSCF.Communication.Scs.Communication;
using HSCF.Communication.Scs.Communication.EndPoints.Tcp;
using HSCF.Communication.ScsServices.Client;
using HomeSeerAPI;
using Scheduler;
using Scheduler.Classes;


namespace AZ_CallerID
{
    class Tools
    {
        string dvPhoneID = "ZMC-" + (ConfigurationManager.AppSettings["PhoneDeviceID"]);
        string LogFile = ConfigurationManager.AppSettings["LogFileName"];
        string PhonebookFilename = ConfigurationManager.AppSettings["PhonebookFileName"];
        string URLNumberChecker = ConfigurationManager.AppSettings["URLNumberChecker"];
        string URLSiteMarker = ConfigurationManager.AppSettings["URLSiteMarker"];
        string Unknown = ConfigurationManager.AppSettings["UnknownCaller"];
        string StartParsingMarker = ConfigurationManager.AppSettings["StartParsingMarker"];
        string EndParsingMarker = ConfigurationManager.AppSettings["EndParsingMarker"];
        string MiddleParsingMarker = ConfigurationManager.AppSettings["MiddleParsingMarker"];

        static IHSApplication hs = new hsapplication();
        static clsHSPI _plugin;
        private const string IFACE_NAME = "AZ_CallerID";
        private static IScsServiceClient<IHSApplication> _client;
        private static IScsServiceClient<IAppCallbackAPI> _clientCallback;


        public Tools()
        {
        }

        public void WriteToLog(string LogText)
        {
            using (StreamWriter stream = File.AppendText(LogFile))
            {
                stream.WriteLine(LogText);
            }
        }

 
        #region Homeseer
        public int HSConnect()
        {
            string HSLocation = ConfigurationManager.AppSettings["HSLocation"];
            int HSPort = Convert.ToInt32(ConfigurationManager.AppSettings["HSPort"]);
            try
            {
                _plugin = new clsHSPI();
                _client = ScsServiceClientBuilder.CreateClient<IHSApplication>(new ScsTcpEndPoint(HSLocation, HSPort), _plugin);
                _clientCallback = ScsServiceClientBuilder.CreateClient<IAppCallbackAPI>(new ScsTcpEndPoint(HSLocation, HSPort), _plugin);

                _client.Connect();
                _clientCallback.Connect();

                hs = _client.ServiceProxy;
                hs.Connect(IFACE_NAME, "");
                return 0;
            }
            catch
            {
                return 1;
            }
        }

        /// <summary>
        /// Control a device
        /// </summary>
        /// <param name="strDeviceName"></param>
        /// <param name="Action"></param>
        public void DoDevice(int Value, string strText)
        {
            try
            {
                var dvRef = hs.GetDeviceRef("ZMC-W77");
                hs.SetDeviceString(dvRef, strText, true);
                hs.SetDeviceValue(dvPhoneID, Value);
            }
            catch
            {
                Console.WriteLine("Device " + dvPhoneID + " isn't avaulable in HS3");
                WriteToLog("Device " + dvPhoneID + " isn't avaulable in HS3");
            }
        }
        #endregion
        /// <summary>
        /// Read phonebook
        /// </summary>
        /// <param name="PhoneNumber"></param>
        /// <returns></returns>
        public string ReadPhonebook(string PhoneNumber)
        {
            string key = string.Format("Nr{0}", PhoneNumber);
            string Name = hs.GetINISetting("PHONEBOOK", key, Unknown, PhonebookFilename);
            return Name;
        }


        /// <summary>
        /// Update phonebook
        /// </summary>
        /// <param name="PhoneNumber"></param>
        /// <returns></returns>
        public void UpdatePhonebook(string PhoneNumber, string Name)
        {
            using (StreamWriter stream = File.AppendText(PhonebookFilename))
            {
                stream.WriteLine(string.Format("{0}={1}", string.Format("Nr{0}", PhoneNumber), Name) );
            }
        }

        
        /// <summary>
        /// FindName
        /// Try to find a name with an existing phone number
        /// </summary>
        /// <param name="PhoneNumber"></param>
        /// <returns>Name</returns>
        public string FindName(string PhoneNumber)
        {
            string Name = Unknown;
            string URL = string.Format("{0}{1}", URLNumberChecker, PhoneNumber);

            string responseFromServer = string.Empty;
            try
            {
                // Create a request for the URL. 
                WebRequest request = WebRequest.Create(URL);
                // Get the response.
                WebResponse response = request.GetResponse();
                // Get the stream containing content returned by the server.
                Stream dataStream = response.GetResponseStream();
                // Open the stream using a StreamReader for easy access.
                StreamReader reader = new StreamReader(dataStream);
                // Read the content.
                responseFromServer = reader.ReadToEnd();
                // Clean up the streams and the response.
                reader.Close();
                response.Close();
            }
            catch
            {
                Name = "ERROR";
                return Name;
            }

            // Now find the name
            string marker = URLSiteMarker;
            if (responseFromServer.Contains(marker))
            {
                // Cut the left part off
                responseFromServer = responseFromServer.Substring(responseFromServer.Length - (responseFromServer.Length - responseFromServer.IndexOf(marker)));
                // New marker
                marker = StartParsingMarker;
                responseFromServer = responseFromServer.Substring(responseFromServer.Length - (responseFromServer.Length - responseFromServer.IndexOf(marker)));
                // New marker
                marker = MiddleParsingMarker;
                responseFromServer = responseFromServer.Substring(responseFromServer.Length + marker.Length - (responseFromServer.Length - responseFromServer.IndexOf(marker)));
                Name = responseFromServer.Substring(0, responseFromServer.IndexOf(EndParsingMarker));
            }

            return Name;
        }
 
    }
}
